package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.checklist.CheckList;
import org.greenstone.gatherer.checklist.Entry;
import org.greenstone.gatherer.gui.GUIManager;
import org.greenstone.gatherer.gui.NumberField;
import org.greenstone.gatherer.util.Utility;
import org.greenstone.gatherer.util.XORToggleButtonGroup;
import org.w3c.dom.*;

public class Preferences
    extends JDialog {

    private CheckList warning_preferences_check_list;
    private JButton apply_button;
    private JButton cancel_button;
    private JButton ok_button;
    private JCheckBox use_proxy_checkbox;
    private JCheckBox workflow_browse;
    private JCheckBox workflow_create;
    private JCheckBox workflow_mirror;
    private JCheckBox workflow_gather;
    private JCheckBox workflow_enrich;
    private JCheckBox workflow_design;
    private JCheckBox workflow_export;
    private JCheckBox workflow_preview;
    private JTabbedPane tab_pane;
    private JTextField proxy_host_field;
    private JTextField proxy_port_field;
    private Preferences self;

    static final Dimension LABEL_SIZE = new Dimension(100, 25);
    static final Dimension ROW_SIZE = new Dimension(380, 25);
    static final Dimension SIZE = new Dimension(400, 320);
    static final String TRUE = "true";

    public Preferences() {
	// Initialize
	super(Gatherer.g_man, true);
	this.self = this;
	setSize(SIZE);
	setTitle("Preferences");
	// Creation
	JPanel content_pane = (JPanel) getContentPane();
	tab_pane = new JTabbedPane();
	tab_pane.add("Preferences.Workflow", createWorkflowPreferences());
	tab_pane.add("Preferences.Connection", createConnectionPreferences());
	tab_pane.add("Preferences.Warnings", createWarningPreferences());

	JPanel button_pane = new JPanel();
	ok_button = new JButton("General.OK");
	apply_button = new JButton("General.Apply");
	cancel_button = new JButton("General.Cancel");
	// Connection
	Gatherer.dictionary.register(this, null, false);
	Gatherer.dictionary.register(tab_pane, null, false);
	Gatherer.dictionary.register(ok_button, null, false);
	Gatherer.dictionary.register(apply_button, null, false);
	Gatherer.dictionary.register(cancel_button, null, false);
	ok_button.addActionListener(new OKButtonListener(true));
	apply_button.addActionListener(new OKButtonListener(false));
	cancel_button.addActionListener(new CancelButtonListener());
	// Layout
	button_pane.setBorder(BorderFactory.createEmptyBorder(5,2,2,2));
	button_pane.setLayout(new GridLayout(1,3,0,5));
	button_pane.add(ok_button);
	button_pane.add(apply_button);
	button_pane.add(cancel_button);

	content_pane.setLayout(new BorderLayout());
	content_pane.add(tab_pane, BorderLayout.CENTER);
	content_pane.add(button_pane, BorderLayout.SOUTH);

	Dimension frame_size = Gatherer.g_man.getSize();
	Point frame_location = Gatherer.g_man.getLocation();
	//setLocation(frame_location.x + ((frame_size.width - SIZE.width) / 2), frame_location.y + ((frame_size.height - SIZE.height)));
	setLocation(((frame_size.width - SIZE.width) / 2), ((frame_size.height - SIZE.height)));

	// Clean up
	frame_location = null;
	frame_size = null;
	cancel_button = null;
	ok_button = null;
	button_pane = null;
	tab_pane = null;
	content_pane = null;
	show();
    }

    private JPanel createConnectionPreferences() {
	boolean currently_enabled = Gatherer.config.get("general.use_proxy", true);
	// Creation
	JPanel connection_pane = new JPanel();
	use_proxy_checkbox = new JCheckBox("Preferences.Connection.Use_Proxy", currently_enabled);
	use_proxy_checkbox.setPreferredSize(ROW_SIZE);
	JPanel proxy_host_pane = new JPanel();
	proxy_host_pane.setPreferredSize(ROW_SIZE);
	JLabel proxy_host_label = new JLabel("Preferences.Connection.Proxy_Host");
	proxy_host_label.setPreferredSize(LABEL_SIZE);
	proxy_host_field = new JTextField(Gatherer.config.getString("general.proxy_host", true));
	proxy_host_field.setEnabled(currently_enabled);
	JPanel proxy_port_pane = new JPanel();
	proxy_port_pane.setPreferredSize(ROW_SIZE);
	JLabel proxy_port_label = new JLabel("Preferences.Connection.Proxy_Port");
	proxy_port_label.setPreferredSize(LABEL_SIZE);
	proxy_port_field = new NumberField(Gatherer.config.getString("general.proxy_port", true));
	proxy_port_field.setEnabled(currently_enabled);

	// Connection
	Gatherer.dictionary.register(use_proxy_checkbox, null, false);
	Gatherer.dictionary.register(proxy_host_label, null, false);
	Gatherer.dictionary.register(proxy_port_label, null, false);
	use_proxy_checkbox.addActionListener(new UseProxyListener());
	// Layout
	proxy_host_pane.setLayout(new BorderLayout());
	proxy_host_pane.add(proxy_host_label, BorderLayout.WEST);
	proxy_host_pane.add(proxy_host_field, BorderLayout.CENTER);

	proxy_port_pane.setLayout(new BorderLayout());
	proxy_port_pane.add(proxy_port_label, BorderLayout.WEST);
	proxy_port_pane.add(proxy_port_field, BorderLayout.CENTER);

	connection_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	connection_pane.add(use_proxy_checkbox);
	connection_pane.add(proxy_host_pane);
	connection_pane.add(proxy_port_pane);

	return connection_pane;
    }

    /** The warning preferences are controlled through a checklist. */
    private JPanel createWarningPreferences() {
	// Retrieve all of the warning preferences settings.
	HashMap warning_preferences = Gatherer.config.getAll("warning\\..*", true);
	warning_preferences_check_list = new CheckList();
	for(Iterator keys = warning_preferences.keySet().iterator(); keys.hasNext(); ) {
	    String property = (String) keys.next();
	    String value = (String) warning_preferences.get(property);
				// Remove 'warning.'
	    String title = Gatherer.dictionary.get(property.substring(8) + ".Title");
	    Entry entry = new Entry(title, value.equalsIgnoreCase(TRUE));
	    entry.setProperty(property);
	    warning_preferences_check_list.addEntry(entry);
	}
	// Creation
	JPanel warning_preferences_pane = new JPanel();
	// Connection
	// Layout
	warning_preferences_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	warning_preferences_pane.setLayout(new BorderLayout());
	warning_preferences_pane.add(new JScrollPane(warning_preferences_check_list), BorderLayout.CENTER);

	return warning_preferences_pane;
    }

    private JPanel createWorkflowPreferences() {
	// Read in the predefined configurations file.
	Vector predefined = new Vector();
	Document predefined_document = Utility.parse("xml/workflows.xml", true);
	Element predefined_element = predefined_document.getDocumentElement();
	NodeList workflow_elements = predefined_element.getElementsByTagName("Workflow");
	for(int i = 0; i < workflow_elements.getLength(); i++) {
	    predefined.add(new WorkflowElementWrapper((Element)workflow_elements.item(i)));
	}
	// Creation
	JPanel workflow_preferences_pane = new JPanel();
	JPanel checklist_pane = new JPanel();
	JLabel title_label = new JLabel("Preferences.Workflow.Title");
	title_label.setPreferredSize(ROW_SIZE);
	XORToggleButtonGroup checkbox_group = new XORToggleButtonGroup();
	workflow_browse  = new JCheckBox("Preferences.Workflow.Browse",  Gatherer.config.get("workflow.browse", false));
	workflow_browse.setPreferredSize(ROW_SIZE);
	workflow_mirror  = new JCheckBox("Preferences.Workflow.Mirror",  Gatherer.config.get("workflow.mirror", false));
	workflow_mirror.setPreferredSize(ROW_SIZE);
	workflow_gather  = new JCheckBox("Preferences.Workflow.Gather",  Gatherer.config.get("workflow.gather", false));
	workflow_gather.setPreferredSize(ROW_SIZE);
	workflow_enrich  = new JCheckBox("Preferences.Workflow.Enrich",  Gatherer.config.get("workflow.enrich", false));
	workflow_enrich.setPreferredSize(ROW_SIZE);
	workflow_design  = new JCheckBox("Preferences.Workflow.Design",  Gatherer.config.get("workflow.design", false));
	workflow_design.setPreferredSize(ROW_SIZE);
	workflow_export  = new JCheckBox("Preferences.Workflow.Export",  Gatherer.config.get("workflow.export", false));
	workflow_export.setPreferredSize(ROW_SIZE);
	workflow_create  = new JCheckBox("Preferences.Workflow.Create",  Gatherer.config.get("workflow.create", false));
	workflow_create.setPreferredSize(ROW_SIZE);
	workflow_preview = new JCheckBox("Preferences.Workflow.Preview", Gatherer.config.get("workflow.preview", false));
	workflow_preview.setPreferredSize(ROW_SIZE);
	JPanel predefined_pane = new JPanel();
	JLabel predefined_label = new JLabel("Preferences.Workflow.Predefined.Label");
	predefined_label.setPreferredSize(new Dimension(150,25));
	JComboBox predefined_combobox = new JComboBox(predefined);
	// Connection
	checkbox_group.add(workflow_mirror);
	checkbox_group.add(workflow_gather);
	checkbox_group.add(workflow_enrich);
	checkbox_group.add(workflow_design);
	checkbox_group.add(workflow_export);
	checkbox_group.add(workflow_create);
	checkbox_group.add(workflow_preview);
	Gatherer.dictionary.register(title_label, null, false);
	Gatherer.dictionary.register(workflow_browse, null, false);
	Gatherer.dictionary.register(workflow_mirror, null, false);
	Gatherer.dictionary.register(workflow_gather, null, false);
	Gatherer.dictionary.register(workflow_enrich, null, false);
	Gatherer.dictionary.register(workflow_design, null, false);
	Gatherer.dictionary.register(workflow_export, null, false);
	Gatherer.dictionary.register(workflow_create, null, false);
	Gatherer.dictionary.register(workflow_preview, null, false);
	Gatherer.dictionary.register(predefined_label, null, false);
	predefined_combobox.addActionListener(new PredefinedActionListener());
	// Layout
	checklist_pane.setLayout(new BoxLayout(checklist_pane, BoxLayout.Y_AXIS));
	checklist_pane.add(title_label);
	if(Gatherer.config.get("workflow.browse", true)) {
	    checklist_pane.add(workflow_browse);
	}
	if(Gatherer.config.get("workflow.mirror", true)) {
	    checklist_pane.add(workflow_mirror);
	}
	if(Gatherer.config.get("workflow.gather", true)) {
	    checklist_pane.add(workflow_gather);
	}
	if(Gatherer.config.get("workflow.enrich", true)) {
	    checklist_pane.add(workflow_enrich);
	}
	if(Gatherer.config.get("workflow.design", true)) {
	    checklist_pane.add(workflow_design);
	}
	if(Gatherer.config.get("workflow.export", true)) {
	    checklist_pane.add(workflow_export);
	}
	if(Gatherer.config.get("workflow.create", true)) {
	    checklist_pane.add(workflow_create);
	}
	if(Gatherer.config.get("workflow.preview", true)) {
	    checklist_pane.add(workflow_preview);
	}
	predefined_pane.setLayout(new BorderLayout());
	predefined_pane.add(predefined_label, BorderLayout.WEST);
	predefined_pane.add(predefined_combobox, BorderLayout.CENTER);

	workflow_preferences_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	workflow_preferences_pane.setLayout(new BorderLayout());
	workflow_preferences_pane.add(checklist_pane, BorderLayout.CENTER);
	workflow_preferences_pane.add(predefined_pane, BorderLayout.SOUTH);

	return workflow_preferences_pane;
    }

    public void dispose() {
	// Deregister dictionary components
	Gatherer.dictionary.deregister(this);
	Gatherer.dictionary.deregister(tab_pane);
	Gatherer.dictionary.deregister(ok_button);
	Gatherer.dictionary.deregister(apply_button);
	Gatherer.dictionary.deregister(cancel_button);
	// Dispose
	super.dispose();
    }

    private class OKButtonListener
	implements ActionListener {
	private boolean close;
	public OKButtonListener(boolean close) {
	    this.close = close;
	}
	public void actionPerformed(ActionEvent event) {
				// Submit the various changes.
				// Connection preferences
	    Gatherer.config.set("general.use_proxy", true, use_proxy_checkbox.isSelected());
	    Gatherer.config.setString("general.proxy_host", true, proxy_host_field.getText());
	    Gatherer.config.setString("general.proxy_port", true, proxy_port_field.getText());
	    Gatherer.setProxy();
				// Warning preferences
	    for(int i = 0; i < warning_preferences_check_list.getEntryCount(); i++) {
		Entry entry = warning_preferences_check_list.get(i);
		Gatherer.config.set(entry.getProperty(), true, entry.isSelected());
	    }
				// Workflow preferences
	    Gatherer.config.set("workflow.browse", false, workflow_browse.isSelected());
	    Gatherer.config.set("workflow.mirror", false, workflow_mirror.isSelected());
	    Gatherer.config.set("workflow.gather", false, workflow_gather.isSelected());
	    Gatherer.config.set("workflow.enrich", false, workflow_enrich.isSelected());
	    Gatherer.config.set("workflow.design", false, workflow_design.isSelected());
	    Gatherer.config.set("workflow.export", false, workflow_export.isSelected());
	    Gatherer.config.set("workflow.create", false, workflow_create.isSelected());
	    Gatherer.config.set("workflow.preview", false, workflow_preview.isSelected());
	    Gatherer.g_man.workflowUpdate("Browser", workflow_browse.isSelected());
	    Gatherer.g_man.workflowUpdate("Mirroring", workflow_mirror.isSelected());
	    Gatherer.g_man.workflowUpdate("Collection", workflow_gather.isSelected());
	    Gatherer.g_man.workflowUpdate("MetaEdit", workflow_enrich.isSelected());
	    Gatherer.g_man.workflowUpdate("Build", workflow_design.isSelected());
	    Gatherer.g_man.workflowUpdate("Export", workflow_export.isSelected());
	    Gatherer.g_man.workflowUpdate("Create", workflow_create.isSelected());
	    Gatherer.g_man.workflowUpdate("Preview", workflow_preview.isSelected());
				// Hide dialog
	    if(close) {
		self.dispose();
	    }
	}
    }

    private class CancelButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    self.dispose();
	}
    }

    private class PredefinedActionListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    JComboBox cb = (JComboBox) event.getSource();
	    WorkflowElementWrapper element = (WorkflowElementWrapper) cb.getSelectedItem();
	    CheckboxUpdater task = new CheckboxUpdater(element);
	    SwingUtilities.invokeLater(task);
	}

	private class CheckboxUpdater
	    implements Runnable {
	    private WorkflowElementWrapper element;
	    public CheckboxUpdater(WorkflowElementWrapper element) {
		this.element = element;
	    }
	    public void run() {
		workflow_browse.setSelected(element.getEnabled("browse"));
		workflow_create.setSelected(element.getEnabled("create"));
		workflow_mirror.setSelected(element.getEnabled("mirror"));
		workflow_gather.setSelected(element.getEnabled("gather"));
		workflow_enrich.setSelected(element.getEnabled("enrich"));
		workflow_design.setSelected(element.getEnabled("design"));
		workflow_export.setSelected(element.getEnabled("export"));
		workflow_preview.setSelected(element.getEnabled("preview"));			
	    }
	}
    }

    private class UseProxyListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    boolean enabled = use_proxy_checkbox.isSelected();
	    Gatherer.config.set("general.use_proxy", true, enabled);
				// Fortunately this is already driven by the event thread.
	    proxy_host_field.setEnabled(enabled);
	    proxy_port_field.setEnabled(enabled);
	}
    }

    private class WorkflowElementWrapper {
	private Element element;
	private String text;
	public WorkflowElementWrapper(Element element) {
	    this.element = element;
	}
	public boolean getEnabled(String name) {
	    boolean result = true;
	    if(element.getAttribute(name).equalsIgnoreCase("false")) {
		result = false;
	    }
	    return result;
	}
	public String toString() {
	    if(text == null) {
		text = element.getFirstChild().getNodeValue(); 
	    }
	    return text;
	}
    }
}
